
#include <windows.h>  
#include <commctrl.h>
#include "newrect.h"  



#if defined (WIN32)
	#define IS_WIN32 TRUE
#else
	#define IS_WIN32 FALSE
#endif

#define IS_NT      IS_WIN32 && (BOOL)(GetVersion() < 0x80000000)
#define IS_WIN32S  IS_WIN32 && (BOOL)(!(IS_NT) && (LOBYTE(LOWORD(GetVersion()))<4))
#define IS_WIN95   (BOOL)(!(IS_NT) && !(IS_WIN32S)) && IS_WIN32

HINSTANCE hInst;   // current instance

LPCTSTR lpszAppName = "MyApp";
LPCTSTR lpszTitle   = "TTM_NEWTOOLRECT"; 


BOOL RegisterWin95( CONST WNDCLASS* lpwc );


int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                      LPTSTR lpCmdLine, int nCmdShow)
{
   MSG      msg;
   HWND     hWnd; 
   WNDCLASS wc;

   wc.style         = CS_HREDRAW | CS_VREDRAW;
   wc.lpfnWndProc   = (WNDPROC)WndProc;       
   wc.cbClsExtra    = 0;                      
   wc.cbWndExtra    = 0;                      
   wc.hInstance     = hInstance;              
   wc.hIcon         = LoadIcon (hInstance, lpszAppName); 
   wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
   wc.lpszMenuName  = lpszAppName;              
   wc.lpszClassName = lpszAppName;              

   if ( IS_WIN95 )
   {
      if ( !RegisterWin95( &wc ) )
         return( FALSE );
   }
   else if ( !RegisterClass( &wc ) )
      return( FALSE );

   hInst = hInstance; 

   hWnd = CreateWindow( lpszAppName, 
                        lpszTitle,    
                        WS_OVERLAPPEDWINDOW, 
                        CW_USEDEFAULT, 0, 
                        CW_USEDEFAULT, 0,  
                        NULL,              
                        NULL,              
                        hInstance,         
                        NULL               
                      );

   if ( !hWnd ) 
      return( FALSE );

   ShowWindow( hWnd, nCmdShow ); 
   UpdateWindow( hWnd );         

   while( GetMessage( &msg, NULL, 0, 0) )   
   {
      TranslateMessage( &msg ); 
      DispatchMessage( &msg );  
   }

   return( msg.wParam ); 
}


BOOL RegisterWin95( CONST WNDCLASS* lpwc )
{
   WNDCLASSEX wcex;

   wcex.style         = lpwc->style;
   wcex.lpfnWndProc   = lpwc->lpfnWndProc;
   wcex.cbClsExtra    = lpwc->cbClsExtra;
   wcex.cbWndExtra    = lpwc->cbWndExtra;
   wcex.hInstance     = lpwc->hInstance;
   wcex.hIcon         = lpwc->hIcon;
   wcex.hCursor       = lpwc->hCursor;
   wcex.hbrBackground = lpwc->hbrBackground;
   wcex.lpszMenuName  = lpwc->lpszMenuName;
   wcex.lpszClassName = lpwc->lpszClassName;

   // Added elements for Windows 95.
   //...............................
   wcex.cbSize = sizeof(WNDCLASSEX);
   wcex.hIconSm = LoadImage(wcex.hInstance, lpwc->lpszClassName, 
                            IMAGE_ICON, 16, 16,
                            LR_DEFAULTCOLOR );
			
   return RegisterClassEx( &wcex );
}


LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
static TOOLINFO ti;
static int      index;

static HWND hToolTip = NULL;


   switch( uMsg )
   {
      case WM_CREATE :
              {
                 InitCommonControls();

                 // Create tool tip control.
                 //.........................
                 hToolTip = CreateWindowEx( 0,
                                            TOOLTIPS_CLASS, NULL, 
                                            TTS_ALWAYSTIP,
                                            CW_USEDEFAULT, 
                                            CW_USEDEFAULT, 
                                            CW_USEDEFAULT, 
                                            CW_USEDEFAULT,
                                            hWnd, NULL, 
                                            hInst, NULL );

                 // Add the tools to the tool tip control.
                 //.......................................
                 ti.cbSize = sizeof( TOOLINFO );
                 ti.hwnd   = hWnd;
                 ti.hinst  = NULL;
                 ti.uFlags = TTF_SUBCLASS;

                 for( index=0; index < 4; index++ )
                 {
                    ti.uId      = index;
                    ti.lpszText = LPSTR_TEXTCALLBACK;

                    ti.rect.top    = 0;
                    ti.rect.bottom = 10;
                    ti.rect.left   = 10*index;
                    ti.rect.right  = 10*(index+1);

                    SendMessage( hToolTip, TTM_ADDTOOL, 0, (LPARAM)&ti );
                 }
              }
              break;

      case WM_SIZE :
              {
                 // Adjust the tool rectangles for the new
                 // section size.
                 //.......................................
                 RECT     rect;

                 ti.cbSize = sizeof( TOOLINFO );
                 ti.hwnd   = hWnd;

                 GetClientRect( hWnd, &rect );

                 for( index = 0; index < 4; index++ )
                 {
                    ti.uId = index;

                    ti.rect.top    = 0;
                    ti.rect.bottom = HIWORD( lParam );
                    ti.rect.left   = (rect.right/4)*index;
                    ti.rect.right  = (rect.right/4)*(index+1);

                    SendMessage( hToolTip, TTM_NEWTOOLRECT, 0, (LPARAM)&ti );
                 }
              }
              break;

      case WM_PAINT  :
              {
                 // Paint the vertical lines to denote the sections.
                 //.................................................
                 PAINTSTRUCT ps;

                 index = 0;

                 BeginPaint( hWnd, &ps );

                 while ( SendMessage( hToolTip, TTM_ENUMTOOLS, 
                         index, (LPARAM)&ti ) )
                 {
                    MoveToEx( ps.hdc, ti.rect.right, ti.rect.top, NULL );
                    LineTo( ps.hdc, ti.rect.right, ti.rect.bottom );
                    index++;
                 }

                 EndPaint( hWnd, &ps );
              }
              break;

      case WM_NOTIFY :
              switch ( ((LPNMHDR)lParam)->code )
              {
                 case TTN_NEEDTEXT :
                      wsprintf( ((LPTOOLTIPTEXT)lParam)->szText, 
                                "Section %d", 
                                ((LPTOOLTIPTEXT)lParam)->hdr.idFrom+1 );
                      break;
              }
              break;


      case WM_COMMAND :
              switch( LOWORD( wParam ) )
              {
                 case IDM_ABOUT :
                        DialogBox( hInst, "AboutBox", hWnd, (DLGPROC)About );
                        break;

                 case IDM_EXIT :
                        DestroyWindow( hWnd );
                        break;
              }
              break;
      
      case WM_DESTROY :
              PostQuitMessage(0);
              break;

      default :
            return( DefWindowProc( hWnd, uMsg, wParam, lParam ) );
   }

   return( 0L );
}


LRESULT CALLBACK About( HWND hDlg,           
                        UINT message,        
                        WPARAM wParam,       
                        LPARAM lParam)
{
   switch (message) 
   {
       case WM_INITDIALOG: 
               return (TRUE);

       case WM_COMMAND:                              
               if (   LOWORD(wParam) == IDOK         
                   || LOWORD(wParam) == IDCANCEL)    
               {
                       EndDialog(hDlg, TRUE);        
                       return (TRUE);
               }
               break;
   }

   return (FALSE); 
}
